use core::panic;

use tokio;
use sqlx::{mysql::{MySqlPool, MySqlRow}, Row};
mod util;
use util::conversion::_timestamp_to_date_string;
use util::structs::{ UserInfo, UserPost};


#[tokio::main]
async fn main() {
    let mysql_db_url = "mysql://root:change_me@172.17.0.2:3306/aimware?allowPublicKeyRetrieval=true&useSSL=false";
    let pool = MySqlPool::connect(mysql_db_url).await.unwrap();

    let starting_steam_id: String = String::from("STEAM_0:0:7659808");
    println!("Recursive lookup! Starting: {}", &starting_steam_id);

    fetch_user(&pool, "4").await;

}

async fn _fetch_last_info(pool: &MySqlPool, lookup_uid: &str) {
    let res = sqlx::query("SELECT * FROM site_user_info WHERE uid = ?")
    .bind(lookup_uid)
    .fetch_all(pool)
    .await;

    // Get newest HWID
    let mut newest_entry = None;
    match res {
        Ok(r)  => {
            match r.len()  {
                0 => None,
                n => Some({
                    newest_entry = Some(&r[n - 1]);

                })
            };
        },
        Err(_e) => {
            panic!()
        }
    }

    
}

async fn fetch_user(pool: &MySqlPool, lookup_uid: &str) -> UserInfo {
    let res = sqlx::query("SELECT * FROM forum_posts WHERE uid = ?")
    .bind(lookup_uid)
    .fetch_one(pool)
    .await;

    match res {
        Ok(r) => {
            println!("User found!");
            return handle_success(r)
        }, 
        Err(_e) => {
            panic!("Corresponding user wasn't found. That's awkward!")
        }
    }

    fn handle_success(v: MySqlRow) -> UserInfo{
        let result_user = UserInfo {
            uid: v.try_get("uid").unwrap(),
            email: v.try_get("email").unwrap(),
            username: v.try_get("username").unwrap(),
            date_reg: v.try_get("regdate").unwrap(),
            date_last_active: v.try_get("lastactive").unwrap(),
            date_last_visit: v.try_get("lastvisit").unwrap(),
            password_hash: v.try_get("password").unwrap(),
            password_salt: v.try_get("salt").unwrap(),
            hardware_fingerprint: String::from(""),
            SIDs: Vec::new()
        };

        println!("Username: {:?}", result_user);

        result_user
    }

}


async fn fetch_user_by_sid(_pool: &MySqlPool, lookup_sid: &str) {
    let _statement = format!("SELECT * FROM site_user_info WHERE sid_last LIKE '%{lookup_sid}%'");

}




async fn _fetch_user_posts(pool: &MySqlPool, lookup_uid: &str) -> Vec<UserPost>{
    let res= sqlx::query("SELECT * FROM forum_posts WHERE uid = ?")
    .bind(lookup_uid)
    .fetch_all(pool)
    .await;

    match res {
        Ok(v) => {
            return handle_success(&v)
        },
        Err(_e) => {
            panic!()
        }
    };
    
    fn handle_success(v: &Vec<MySqlRow>) -> Vec<UserPost> {
        let mut posts: Vec<UserPost> = vec![];
        for row in v {
            let post = UserPost {
                post_id: row.try_get("pid").unwrap(),
                post_content: row.try_get::<String, &str>("message").unwrap().to_string(),
                post_timestamp: row.try_get("dateline").unwrap(),
                poster_uid: row.try_get("uid").unwrap(),
            };
            println!("PosterUID: {} | PostID: {} | UnixTimestamp: {}", &post.poster_uid, &post.post_id.to_string(), _timestamp_to_date_string(&post.post_timestamp));
            posts.push(post);
        }
        posts
    }
}
